# -*- coding: utf-8 -*-

"""
upraise.routing
~~~~~~~~~~~~~~~

URL dispatching and generation.

:date: $Date: 2007-07-20 22:20:04 +0200 (Fr, 20 Jul 2007) $
:revision: $Rev: 161 $
:copyright: 2006-2007 Jochen Kupperschmidt
:license: MIT, see LICENSE for details.
"""

from urllib import urlencode
from urlparse import urlunsplit

from paste.request import path_info_pop

from upraise.util import request


def url_for(controller, action='index', id=None, qualified=False, **other):
    """Generate the corresponding URL, similar to what routes.url_for() does.

    This function is very limited, but allows for easier switching to Routes if
    that becomes necessary in the future.
    """
    url_parts = ['', controller]
    if id is not None or action != 'index':
        url_parts.append(action)
        if id is not None:
            url_parts.append(str(id))
    url = '/'.join(url_parts)

    query = urlencode(other)
    if qualified:
        url = urlunsplit((request.scheme, request.host, url, query, ''))
    elif other:
        url += '?' + query
    return url


# ---------------------------------------------------------------- #
# middleware

class RoutingMiddleware(object):
    """A middleware that does simple URL path dispatching.

    The path is interpreted as ``/controller/action/id``.

    Implements the `wsgiorg.routing_args specification`__.

    __ http://wsgi.org/wsgi/Specifications/routing_args
    """

    def __init__(self, app, default_controller=None):
        self.app = app
        self.default_controller = default_controller

    def __call__(self, environ, start_response):
        args = [self.default_controller or None, 'index', None]
        for i in range(len(args)):
            segment = path_info_pop(environ)
            if segment:
                args[i] = segment

        controller, action, id = args
        named_args = dict(controller=controller, action=action)
        if id is not None:
            named_args.update(id=id)

        environ['wsgiorg.routing_args'] = ((), named_args)
        return self.app(environ, start_response)


def make_middleware(app, global_conf, default_controller=None):
    """Add the routing middleware to the application stack.

    This is an entry point for Paste Deploy (egg:Upraise#routing).
    """
    return RoutingMiddleware(app, default_controller=default_controller)
